/**
 *******************************************************************************
 * @file    system_TMPM411_unitA.c
 * @brief   CMSIS Cortex-M4 Device Peripheral Access Layer Source File for the
 *          TOSHIBA 'TMPM411_unitA' Device Series 
 * @version V2.0.2.1
 * @date    2014/07/08
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2014 All rights reserved
 *
 *                                                              modified by Keil
 *******************************************************************************
 */

#include "TMPM411_unitA.h"

/*-------- <<< Start of configuration section >>> ----------------------------*/

/* Watchdog Timer (WD) Configuration */
#define WD_SETUP                (1U)
#define WDMOD_Val               (0x00000000UL)
#define WDCR_Val                (0x000000B1UL)

/* Clock Generator (CG) Configuration */
#define CLOCK_SETUP             (1U)
#define CLKCR_Val               (0x00000000UL)
#define OSCSEL_Val              (0x00000002UL) /* Select fPLL as high-speed clock (fc) */
#define OSCEN_Val               (0x00000002UL) 

#define STBYCR_Val              (0x00000000UL)

#define CG_8M_MUL_6_FPLL        (0x0000D22EUL)
#define CG_8M_MUL_8_FPLL        (0x0000D23EUL)
#define CG_8M_MUL_9_FPLL        (0x0000D244UL)
#define CG_8M_MUL_10_FPLL       (0x0000D24CUL)

#define CG_10M_MUL_8_FPLL       (0x0000D23CUL)

#define CG_12M_MUL_4_FPLL       (0x0000D21EUL)
#define CG_12M_MUL_6_FPLL       (0x0000D22EUL)

#define CG_16M_MUL_3_FPLL       (0x0000D216UL)
#define CG_16M_MUL_4_FPLL       (0x0000D21EUL)
#define CG_16M_MUL_5_FPLL       (0x0000D224UL)

#define CG_OSCEN_EHOSCEN_CLEAR               ((uint32_t)0xFFFFFFF9)
#define CG_OSCEN_EHOSCEN_EHOSC               ((uint32_t)0x00000002)
#define CG_OSCEN_EHOSCEN_EHCLKIN             ((uint32_t)0x00000004)

#define CG_WUHCR_WUPT_CLEAR                  ((uint32_t)0xF000FFFF)

#define CG_OSCSEL_OSCSEL_PLL                 ((uint32_t)0x00000002)
#define CG_OSCSTF_OSCF_PLL                   ((uint32_t)0x00000002)
#define CG_OSCSTF_OSCF_IHOSC                 ((uint32_t)0x00000000)
#define CG_OSCSTF_OSCF_EHOSCEN               ((uint32_t)0x00000001)
    
#define WD_MOD_WDTE_SET                      ((uint32_t)0x00000080)

#define CG_CLKCR_GEAR_CLEAR                  ((uint32_t)0xFFFFFFFC)
#define CG_CLKCR_GEAR_0                      ((uint32_t)0x00000000)

#define PLLSET_Val             CG_16M_MUL_4_FPLL
#define PLLEN_Val              (0x00000001UL)

/*-------- <<< End of configuration section >>> ------------------------------*/

/*-------- DEFINES -----------------------------------------------------------*/
/* Define clocks */
#define OSC_8M                  ( 8000000UL)
#define OSC_10M                 (10000000UL)
#define OSC_12M                 (12000000UL)
#define OSC_16M                 (16000000UL)
#define EXTALH                  OSC_16M     /* External high-speed oscillator freq */
#define EHCLKIN                 OSC_16M     /* External clcok input freq */
#define XTALH                   OSC_10M     /* Internal high-speed oscillator freq */

/* Configure Warm-up time */
#define HZ_1M                  (1000000UL)
#define WU_TIME_EXT            (5000UL)          /* warm-up time for EXT is 5ms   */
#define WU_TIME_PLL            (100UL)           /* warm-up time for PLL is 100us */
#define WUHCR_WUPT_EXT         ((WU_TIME_EXT * EXTALH / HZ_1M /16UL) << 16U) /* CG0WUHCR<WUPT[11:0]> = warm-up time(us) * EXTALH / 16 */
#define WUHCR_WUPT_PLL         ((WU_TIME_PLL * EXTALH / HZ_1M /16UL) << 16U)

#if (CLOCK_SETUP)               /* Clock(external) Setup */
/* Determine core clock frequency according to settings */
/* System clock is high-speed clock*/
#if (OSCSEL_Val & 0x1UL)
  #if (OSCEN_Val & 0x2UL)
   #define __CORE_CLK (EXTALH)
  #elif (OSCEN_Val & 0x4UL)
   #define __CORE_CLK (EHCLKIN)
  #else  
   #define __CORE_CLK (0U)
  #endif
#elif (!(OSCSEL_Val & 0x3UL))
  #if (OSCEN_Val & 0x1UL)
   #define __CORE_CLK (XTALH)
  #else  
   #define __CORE_CLK (0U)
  #endif
#elif (OSCSEL_Val & 0x2UL)
  #if (OSCEN_Val & 0x2UL)
   #define CORE_TALH (EXTALH)
  #elif (OSCEN_Val & 0x4UL)
   #define CORE_TALH (EHCLKIN)
  #else  
   #define CORE_TALH (0U)
  #endif
  #if (PLLEN_Val & 1U)   /* If PLL is enabled */
    #if (CORE_TALH == OSC_8M)                             /* If input is 8MHz */
      #if (PLLSET_Val == (CG_8M_MUL_6_FPLL))
        #define __CORE_CLK   (CORE_TALH * 6U )            /* output clock is 48MHz */
      #elif (PLLSET_Val == (CG_8M_MUL_8_FPLL))
        #define __CORE_CLK   (CORE_TALH * 8U )            /* output clock is 64MHz */
      #elif (PLLSET_Val == (CG_8M_MUL_9_FPLL))
        #define __CORE_CLK   (CORE_TALH * 9U )            /* output clock is 72MHz */      
      #elif (PLLSET_Val == (CG_8M_MUL_10_FPLL))
        #define __CORE_CLK   (CORE_TALH * 10U )           /* output clock is 80MHz */          
      #else                                               /* fc -> reserved */
        #define __CORE_CLK   (0U)
      #endif                                              /* End input is 8MHz */                                                                                                                                                                                                  
    #elif (CORE_TALH == OSC_10M)                       /* If input is 10MHz */
      #if (PLLSET_Val == (CG_10M_MUL_8_FPLL))
        #define __CORE_CLK   (CORE_TALH * 8U )            /* output clock is 80MHz */                            
      #else                                               /* fc -> reserved */
        #define __CORE_CLK   (0U)
      #endif                                              /* End input is 10MHz */   
    #elif (CORE_TALH == OSC_12M)                       /* If input is 12MHz */
      #if (PLLSET_Val == CG_12M_MUL_4_FPLL)
        #define __CORE_CLK   (CORE_TALH * 4U )            /* output clock is 48MHz */
      #elif (PLLSET_Val == CG_12M_MUL_6_FPLL)
        #define __CORE_CLK   (CORE_TALH * 6U )            /* output clock is 72MHz */ 
      #else                                               /* fc -> reserved */
        #define __CORE_CLK   (0U)
      #endif                                              /* End input is 12MHz */    
    #elif (CORE_TALH == OSC_16M)                       /* If input is 16MHz */
      #if (PLLSET_Val == CG_16M_MUL_3_FPLL)
        #define __CORE_CLK   (CORE_TALH * 3U )            /* output clock is 48MHz */
      #elif (PLLSET_Val == CG_16M_MUL_4_FPLL)
        #define __CORE_CLK   (CORE_TALH * 4U )            /* output clock is 64MHz */
      #elif (PLLSET_Val == CG_16M_MUL_5_FPLL)
        #define __CORE_CLK   (CORE_TALH * 5U )            /* output clock is 80MHz */
      #else                                               /* fc -> reserved        */
        #define __CORE_CLK   (0U)
      #endif                                              /* End input is 16MHz    */
    #else                                                 /* input clock not known */
      #define __CORE_CLK   (0U)
      #error "Core Oscillator Frequency invalid!"
    #endif                                                /* End switch input clock */
  #else
    #define __CORE_CLK   (0U)
  #endif
#endif

#if ((CLKCR_Val & 7U) == 0U)                          /* Gear -> fc             */
  #define __CORE_SYS   (__CORE_CLK)
#elif ((CLKCR_Val & 7U) == 1U)                          /* Gear -> fc/2           */
  #define __CORE_SYS   (__CORE_CLK / 2U)
#elif ((CLKCR_Val & 7U) == 2U)                          /* Gear -> fc/4           */
  #define __CORE_SYS   (__CORE_CLK / 4U)
#elif ((CLKCR_Val & 7U) == 3U)                          /* Gear -> fc/8           */
  #define __CORE_SYS   (__CORE_CLK / 8U)
#elif ((CLKCR_Val & 7U) == 4U)                          /* Gear -> fc/16          */
  #define __CORE_SYS   (__CORE_CLK / 16U)
#else                                                   /* Gear -> reserved       */
  #define __CORE_SYS   (0U)
#endif

#else
  #define __CORE_SYS   (XTALH)
  
#endif                                                  /* clock Setup */

/* Clock Variable definitions */
uint32_t SystemCoreClock = __CORE_SYS;  /*!< System Clock Frequency (Core Clock) */

/**
 * Initialize the system
 *
 * @param  none
 * @return none
 *
 * @brief  Update SystemCoreClock according register values.
 */
void SystemCoreClockUpdate(void)
{
    uint32_t CoreClock = 0U;
    uint32_t CoreClockInput = 0U;

    /* Determine clock frequency according to clock register values */
    /* System clock is high-speed clock */
    switch (TSB_CG0->OSCSTF) {
        case CG_OSCSTF_OSCF_IHOSC:
            CoreClock = XTALH;    /* fc source is Internal high-speed oscillator */
            break;
        case CG_OSCSTF_OSCF_EHOSCEN:
            if (TSB_CG0->OSCEN & CG_OSCEN_EHOSCEN_EHOSC) {
                CoreClock = EXTALH;  /* fc source is External high-speed oscillator (EHOSC) */
            } else if (TSB_CG0->OSCEN & CG_OSCEN_EHOSCEN_EHCLKIN) {
                CoreClock = EHCLKIN;  /* fc source is external clock input (EHCLKIN) */
            } else {
                /* Do nothing */
            }
            break;
        case CG_OSCSTF_OSCF_PLL:
            /* fc source is PLL clock output, so it should be calculated latter. */
            if (TSB_CG0->OSCEN & CG_OSCEN_EHOSCEN_EHOSC) {
                CoreClock = EXTALH;  /* fc source is External high-speed oscillator (EHOSC) */
            } else if (TSB_CG0->OSCEN & CG_OSCEN_EHOSCEN_EHCLKIN) {
                CoreClock = EHCLKIN;  /* fc source is external clock input (EHCLKIN) */
            } else {
                /* Do nothing */
            }
            break;
        default:
            /* Do nothing */
            break;
    }

    if ((TSB_CG0->OSCSTF == CG_OSCSTF_OSCF_PLL) && (TSB_CG0_PLLEN_PLLON == 1U)) {  /* If PLL is the fc source */
        if (CoreClock == OSC_8M) {                                          /* If input is 8MHz */
            if (TSB_CG0->PLLSET == CG_8M_MUL_6_FPLL) {     
                CoreClockInput = CoreClock * 6U;                             /* output clock is 48MHz */
            } else if (TSB_CG0->PLLSET == CG_8M_MUL_8_FPLL) {
                CoreClockInput = CoreClock * 8U;                            /* output clock is 64MHz */             
            } else if (TSB_CG0->PLLSET == CG_8M_MUL_9_FPLL) {
                CoreClockInput = CoreClock * 9U;                            /* output clock is 72MHz */             
            } else if (TSB_CG0->PLLSET == CG_8M_MUL_10_FPLL) {
                CoreClockInput = CoreClock * 10U;                            /* output clock is 80MHz */                         
            } else {
                CoreClockInput = 0U;                                         /* fc -> reserved            */
            }
        } else if (CoreClock == OSC_10M) {                                  /* If input is 10MHz */
            if (TSB_CG0->PLLSET == CG_10M_MUL_8_FPLL) {
                CoreClockInput = CoreClock * 8U;                             /* output clock is 80MHz */
            } else {
                CoreClockInput = 0U;                                         /* fc -> reserved */
            }
        } else if (CoreClock == OSC_12M) {                                  /* If input is 12MHz */
            if ((TSB_CG0->PLLSET) == CG_12M_MUL_4_FPLL) {
                CoreClockInput = CoreClock * 4U;                             /* output clock is 48MHz */
            } else if ((TSB_CG0->PLLSET) == CG_12M_MUL_6_FPLL) {
                CoreClockInput = CoreClock * 6U;                             /* output clock is 72MHz */             
            } else {
                CoreClockInput = 0U;                                         /* fc -> reserved */
            }
        } else if (CoreClock == OSC_16M) {                                  /* If input is 16MHz */
            if ((TSB_CG0->PLLSET) == CG_16M_MUL_3_FPLL) {
                CoreClockInput = CoreClock * 3U;                             /* output clock is 48MHz */
            } else if ((TSB_CG0->PLLSET) == CG_16M_MUL_4_FPLL) {
                CoreClockInput = CoreClock * 4U;                             /* output clock is 64MHz */
            } else if ((TSB_CG0->PLLSET) == CG_16M_MUL_5_FPLL) {
                CoreClockInput = CoreClock * 5U;                             /* output clock is 80MHz */
            } else {
                CoreClockInput = 0U;                                         /* fc -> reserved        */
            }
        }    
    } else {
       CoreClockInput = CoreClock; 
    }
 
    switch (TSB_CG0->CLKCR & 7U) {
    case 0U:
        SystemCoreClock = CoreClockInput;                                    /* Gear -> fc  */
        break;                                                              
    case 1U:                                                                 /* Gear -> fc/2 */
        SystemCoreClock = CoreClockInput / 2U;
        break;
    case 2U:                                                                 /* Gear -> fc/4 */
        SystemCoreClock = CoreClockInput / 4U;
        break;
    case 3U:                                                                 /* Gear -> fc/8 */
        SystemCoreClock = CoreClockInput / 8U;
        break;
    case 4U:                                                                 /* Gear -> fc/16 */
        SystemCoreClock = CoreClockInput / 16U;
        break;
    default:
        SystemCoreClock = 0U;
        break;
    }
}

/**
 * Initialize the system
 *
 * @param  none
 * @return none
 *
 * @brief  Setup the microcontroller system.
 *         Initialize the System.
 */
void SystemInit(void)
{
                                             /* added by Keil */
#if (__FPU_USED == 1)                        /* __FPU_USED is defined in core_cm4.h */
  /* enable FPU if available and used */
  SCB->CPACR |= ((3UL << 10*2) |             /* set CP10 Full Access                */
                 (3UL << 11*2)  );           /* set CP11 Full Access                */
#endif

#if (WD_SETUP)                  /* Watchdog Setup */
    while (TSB_WD0->FLG != 0U) {
    }                /* When writing to WDMOD or WDCR, confirm "0" of WDFLG<FLG>. */
    TSB_WD0->MOD = WDMOD_Val;
    if (!TSB_WD0_MOD_WDTE) {     /* If watchdog is to be disabled */
        TSB_WD0->CR = WDCR_Val;
    } else {
        /*Do nothing*/
    }
#endif

#if (CLOCK_SETUP)               /* Clock(external) Setup */
    /* To supply the clock to Unit B. */
    TSB_CPUIF0_CKCR_CK = 1U;
    while (!TSB_CPUIF0_CKCR_CK) {
    }                          /* Confirm CPUIF0CKCR<CK> is "1" (supplying the clock). */
    TSB_CPUIF0_RSTCR_RSTREL = 1U;

    /* The external high-speed oscillator(EHOSC) is used. */
    TSB_CG0->OSCEN &= CG_OSCEN_EHOSCEN_CLEAR;
    TSB_CG0->OSCEN |= CG_OSCEN_EHOSCEN_EHOSC;
    TSB_CG0_WUHCR_WUCLK = 1U;
    TSB_CG0->WUHCR &= CG_WUHCR_WUPT_CLEAR;
    TSB_CG0->WUHCR |= WUHCR_WUPT_EXT;
    TSB_CG0_WUHCR_WUON = 1U;
    while (TSB_CG0_WUHCR_WUEF) {
    }                           /* Warm-up */

    /* PLL operation setting. */
    TSB_CG0_PLLEN_PLLON = 0U;
    TSB_CG0->PLLSET = CG_16M_MUL_5_FPLL;
    TSB_CG0_PLLEN_PLLON = 1U;
    TSB_CG0->STBYCR = STBYCR_Val;
    TSB_CG0_WUHCR_WUCLK = 1U;
    TSB_CG0->WUHCR &= CG_WUHCR_WUPT_CLEAR;
    TSB_CG0->WUHCR |= WUHCR_WUPT_PLL;
    TSB_CG0_WUHCR_WUON = 1U;
    while (TSB_CG0_WUHCR_WUEF) {
    }                           /* Warm-up */

    /* Set a division ratio of the clock gear. */
    TSB_CG0->OSCSEL = CG_OSCSEL_OSCSEL_PLL;
    while (TSB_CG0->OSCSTF != CG_OSCSTF_OSCF_PLL) {
    }                         /* Confirm the source clock for fc is changed to the fPLL */    
    TSB_CG0->CLKCR &= CG_CLKCR_GEAR_CLEAR;
    TSB_CG0->CLKCR |= CG_CLKCR_GEAR_0;
#endif
}
