/**
 *******************************************************************************
 * @file    system_TMPM462.c
 * @brief   CMSIS Cortex-M4 Device Peripheral Access Layer Source File for the
 *          TOSHIBA 'TMPM462' Device Series 
 * @version V2.0.2.1
 * @date    2013/08/14
 * 
 * THE SOURCE CODE AND ITS RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA
 * CORPORATION MAKES NO OTHER WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR,
 * STATUTORY AND DISCLAIMS ANY AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY,
 * SATISFACTORY QUALITY, NON INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * THE SOURCE CODE AND DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION
 * RESERVES THE RIGHT TO INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER
 * REVISIONS OF IT, AND TO MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR
 * THE PRODUCTS OR TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME.
 * 
 * TOSHIBA CORPORATION SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT OR
 * CONSEQUENTIAL DAMAGE OR LIABILITY ARISING FROM YOUR USE OF THE SOURCE CODE OR
 * ANY DOCUMENTATION, INCLUDING BUT NOT LIMITED TO, LOST REVENUES, DATA OR
 * PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL OR CONSEQUENTIAL NATURE, PUNITIVE
 * DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS ARISING OUT OF OR IN CONNECTION
 * WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF ADVISED OF THE POSSIBILITY OR
 * PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM FOR SUCH DAMAGE IS BASED UPON
 * WARRANTY, CONTRACT, TORT, NEGLIGENCE OR OTHERWISE.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2013 All rights reserved
 *******************************************************************************
 */

#include "TMPM462.h"

/*-------- <<< Start of configuration section >>> ----------------------------*/

/* Watchdog Timer (WD) Configuration */
#define WD_SETUP                1U
#define WDMOD_Val               (0x00000000UL)
#define WDCR_Val                (0x000000B1UL)

/* Clock Generator (CG) Configuration */
#define CLOCK_SETUP             1U
/*SYSCR*/
#define SYSCR_Val               (0x00000000UL)
/*OSCCR*/
#define OSCSEL_BITNUM           8U                          /*OSCSEL bit number*/
#define XEN1_1                  (0x1U<<0U)                  /*Bit0:External high-speed oscillator operation is USED*/
#define XEN2_1                  (0x1U<<1U)                  /*Bit1:Internal high-speed oscillator operation (for SYS) is oscillator*/
#define OSCSEL_1                (0x1U<<OSCSEL_BITNUM)       /*Bit8:Select exernal high-speed OSC*/
#define HOSCON_1                (0x1U<<10U)                 /*Bit10:Select external OSC source as exernal OSC*/
#define OSCCR_Val               (0x00000501UL)              /*(XEN1_1|OSCSEL_1|HOSCON_1)*/

#define WUEON_1                 (0x1U<<14U)                 /*warm-up timer start*/
#define WUEF_1                  (0x1U<<15U)                 /*Status of warm-up timer, 0:finished*/
#define WUPSEL1_0               ((uint32_t)(~(0x1U<<16U)))
#define WUPSEL2_1               (0x1U<<17U)
/*STBYCR*/
#define STBY_STOP2              0x03U
#define STBYCR_Val              ((uint32_t)STBY_STOP2)
/*PLLSEL*/
#define PLL_USE_BITNUM          17U                     /*PLLSEL bit number*/
#define PLLON_BITNUM            16U                     /*PLLON bit number*/
#define PLLON_1                 ((0x01U<<PLLON_BITNUM))    /*Bit16:PLL is ON*/
#define PLLSEL_1                ((0x01U<<PLL_USE_BITNUM))
#define PLLSEL_PLLON            ((uint32_t)PLLON_1)
#define PLLSEL_PLL_MASK         (0x00070000UL)

#define CG_8M_MUL_4_FPLL        (0x00006ADFUL<<1U)
#define CG_8M_MUL_6_FPLL        (0x00006917UL<<1U)
#define CG_8M_MUL_8_FPLL        (0x0000691FUL<<1U)
#define CG_8M_MUL_10_FPLL       (0x00006A26UL<<1U)
#define CG_8M_MUL_12_FPLL       (0x0000692EUL<<1U)

#define CG_10M_MUL_4_FPLL        (0x00006A0FUL<<1U)
#define CG_10M_MUL_5_FPLL        (0x00006A13UL<<1U)
#define CG_10M_MUL_6_FPLL        (0x00006917UL<<1U)
#define CG_10M_MUL_10_FPLL       (0x00006A26UL<<1U)
#define CG_10M_MUL_12_FPLL       (0x0000691EUL<<1U)

#define CG_12M_MUL_4_FPLL        (0x00006A0FUL<<1U)
#define CG_12M_MUL_10_FPLL       (0x00006A26UL<<1U)

#define CG_16M_MUL_4_FPLL        (0x00006A0FUL<<1U)
#define CG_16M_MUL_6_FPLL        (0x00006A16UL<<1U)
                                       
#define PLLSEL_Ready            CG_10M_MUL_12_FPLL

#define PLLSEL_Val              (PLLSEL_Ready|PLLSEL_1)
#define PLLSEL_MASK             0x0000FFFEUL

/*-------- <<< End of configuration section >>> ------------------------------*/

/*-------- DEFINES -----------------------------------------------------------*/
/* Define clocks */
#define OSC_8M                  ( 8000000UL)
#define OSC_10M                 (10000000UL)
#define OSC_12M                 (12000000UL)
#define OSC_16M                 (16000000UL)

#define EXTALH                  OSC_10M         /* External high-speed oscillator freq */
#define XTALH                   OSC_10M         /* Internal high-speed oscillator freq */

/* Configure Warm-up time */
#define HZ_1M                   1000000UL
#define WU_TIME_EXT             5000UL          /* warm-up time for EXT is 5ms   */
#define WU_TIME_PLL             100UL           /* warm-up time for PLL is 100us */
#define OSCCR_WUODR_MASK        (0x000FFFFFUL)
#define OSCCR_WUODR_EXT         ((WU_TIME_EXT * EXTALH / HZ_1M /16U) << 20U) /* OSCCR<WUPODR11:0> = warm-up time(us) * EXTALH(MHz) / 16 */
#define OSCCR_WUODR_PLL         ((WU_TIME_PLL * EXTALH / HZ_1M /16U) << 20U)

#if (CLOCK_SETUP)               /* Clock(external) Setup */
/* Determine core clock frequency according to settings */
/* System clock is high-speed clock*/
#if (OSCCR_Val & (1U<<OSCSEL_BITNUM))
  #define CORE_TALH (EXTALH)
#else
  #define CORE_TALH (XTALH)
#endif

#if ((PLLSEL_Val & (1U<<PLL_USE_BITNUM)) && (PLLSEL_Val & (1U<<PLLON_BITNUM)))      /* If PLL selected and enabled */
  #if (CORE_TALH == OSC_8M)                            			/* If input is 8MHz */
    #if ((PLLSEL_Val & PLLSEL_MASK) == (CG_8M_MUL_4_FPLL))
      #define __CORE_CLK_FPLL   (CORE_TALH * 4U )            /* output clock is 32MHz */
    #elif ((PLLSEL_Val & PLLSEL_MASK) == (CG_8M_MUL_6_FPLL))
      #define __CORE_CLK_FPLL   (CORE_TALH * 6U )           /* output clock is 48MHz */
    #elif ((PLLSEL_Val & PLLSEL_MASK) == (CG_8M_MUL_8_FPLL))
      #define __CORE_CLK_FPLL   (CORE_TALH * 8U )           /* output clock is 64MHz */
    #elif ((PLLSEL_Val & PLLSEL_MASK) == (CG_8M_MUL_10_FPLL))
      #define __CORE_CLK_FPLL   (CORE_TALH * 10U )           /* output clock is 80MHz */     
    #elif ((PLLSEL_Val & PLLSEL_MASK) == (CG_8M_MUL_12_FPLL))                            
      #define __CORE_CLK_FPLL   (CORE_TALH * 12U )           /* output clock is 96MHz */  
    #else                                                    /* fc -> reserved */
      #define __CORE_CLK_FPLL   (0U)
    #endif                                                   /* End input is 8MHz */
  #elif (CORE_TALH == OSC_10M)                        			 /* If input is 10MHz */
    #if ((PLLSEL_Val & PLLSEL_MASK) == (CG_10M_MUL_4_FPLL))
      #define __CORE_CLK_FPLL   (CORE_TALH * 4U )            /* output clock is 40MHz */
    #elif ((PLLSEL_Val & PLLSEL_MASK) == (CG_10M_MUL_5_FPLL))
      #define __CORE_CLK_FPLL   (CORE_TALH * 5U )           /* output clock is 50MHz */
    #elif ((PLLSEL_Val & PLLSEL_MASK) == (CG_10M_MUL_6_FPLL))
      #define __CORE_CLK_FPLL   (CORE_TALH * 6U )           /* output clock is 60MHz */
    #elif ((PLLSEL_Val & PLLSEL_MASK) == (CG_10M_MUL_10_FPLL))
      #define __CORE_CLK_FPLL   (CORE_TALH * 10U )           /* output clock is 100MHz */      
    #elif ((PLLSEL_Val & PLLSEL_MASK) == (CG_10M_MUL_12_FPLL))                               
      #define __CORE_CLK_FPLL   (CORE_TALH * 12U )           /* output clock is 120MHz */ 
    #else                                                    /* fc -> reserved */
      #define __CORE_CLK   (0U)
    #endif                                                   /* End input is 10MHz */
  #elif (CORE_TALH == OSC_12M)                        			 /* If input is 12MHz */                         
    #if ((PLLSEL_Val & PLLSEL_MASK) == CG_12M_MUL_4_FPLL)                                              
      #define __CORE_CLK_FPLL   (CORE_TALH * 4U )            /* output clock is 48MHz */              
    #elif ((PLLSEL_Val & PLLSEL_MASK) == CG_12M_MUL_10_FPLL)                                           
      #define __CORE_CLK_FPLL   (CORE_TALH * 10U )           /* output clock is 120MHz */              
    #else                                                    /* fc -> reserved */                      
      #define __CORE_CLK_FPLL   (0U)                                                                   
    #endif                                                   /* End input is 12MHz */               
   #elif (CORE_TALH == OSC_16M)                         /* If input is 16MHz */
    #if ((PLLSEL_Val & PLLSEL_MASK) == CG_16M_MUL_4_FPLL)
      #define __CORE_CLK_FPLL   (CORE_TALH * 4U )            /* output clock is 64MHz */
    #elif ((PLLSEL_Val & PLLSEL_MASK) == CG_16M_MUL_6_FPLL)
      #define __CORE_CLK_FPLL   (CORE_TALH * 6U )           /* output clock is 96MHz */
    #else                                                    /* fc -> reserved */
      #define __CORE_CLK_FPLL   (0U)
    #endif                                                   /* End input is 16MHz */
  #else                                                /* input clock not known */
    #define __CORE_CLK_FPLL   (0U)
    #error "Core Oscillator Frequency invalid!"
  #endif                                                /* End switch input clock */
#else
  #define __CORE_CLK   (CORE_TALH)
#endif

#if   ((SYSCR_Val & 7U) == 0U)                          /* Gear -> fc             */
  #define __CORE_SYS   (__CORE_CLK)
#elif ((SYSCR_Val & 7U) == 4U)                          /* Gear -> fc/2           */
  #define __CORE_SYS   (__CORE_CLK / 2U)
#elif ((SYSCR_Val & 7U) == 5U)                          /* Gear -> fc/4           */
  #define __CORE_SYS   (__CORE_CLK / 4U)
#elif ((SYSCR_Val & 7U) == 6U)                          /* Gear -> fc/8           */
  #define __CORE_SYS   (__CORE_CLK / 8U)
#elif ((SYSCR_Val & 7U) == 7U)                          /* Gear -> fc/16          */
  #define __CORE_SYS   (__CORE_CLK / 16U)
#else                                                   /* Gear -> reserved       */
  #define __CORE_SYS   (0U)
#endif

#else
  #define __CORE_SYS   (XTALH)
#endif                                                  /* clock Setup */

/* Clock Variable definitions */
uint32_t SystemCoreClock = __CORE_SYS;  /*!< System Clock Frequency (Core Clock) */


/**
 * Initialize the system
 *
 * @param  none
 * @return none
 *
 * @brief  Update SystemCoreClock according register values.
 */
void SystemCoreClockUpdate(void)
{                               /* Get Core Clock Frequency */
    uint32_t CoreClock = 0U;
    uint32_t CoreClockInput = 0U;
    /* Determine clock frequency according to clock register values */
    /* System clock is high-speed clock */
    if ((TSB_CG->OSCCR&OSCSEL_1)) {  /* If system clock is External high-speed oscillator freq */
        CoreClock = EXTALH;
    } else {                    /* If system clock is Internal high-speed oscillator freq */
        CoreClock = XTALH;
    }
    if ((TSB_CG->PLLSEL&PLLSEL_PLL_MASK) != 0U) {                        /* If PLL enabled  */
        if (CoreClock == OSC_8M) {                                          /* If input is 8MHz */
            if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_8M_MUL_4_FPLL) {     
                CoreClockInput = CoreClock * 4U;                             /* output clock is 32MHz */
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_8M_MUL_6_FPLL) {
                CoreClockInput = CoreClock * 6U;                            /* output clock is 48MHz */                
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_8M_MUL_8_FPLL) {
                CoreClockInput = CoreClock * 8U;                            /* output clock is 64MHz */                
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_8M_MUL_10_FPLL) {
                CoreClockInput = CoreClock * 10U;                            /* output clock is 80MHz */                
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_8M_MUL_12_FPLL) {
                CoreClockInput = CoreClock * 12U;                            /* output clock is 96MHz */
            } else {
                CoreClockInput = 0U;                                         /* fc -> reserved            */
            }
        } else if (CoreClock == OSC_10M) {                                  /* If input is 10MHz */
            if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_10M_MUL_4_FPLL) {     
                CoreClockInput = CoreClock * 4U;                             /* output clock is 40MHz */
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_10M_MUL_5_FPLL) {
                CoreClockInput = CoreClock * 5U;                            /* output clock is 50MHz */                
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_10M_MUL_6_FPLL) {
                CoreClockInput = CoreClock * 6U;                            /* output clock is 60MHz */                
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_10M_MUL_10_FPLL) {
                CoreClockInput = CoreClock * 10U;                            /* output clock is 100MHz */               
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_10M_MUL_12_FPLL) {
                CoreClockInput = CoreClock * 12U;                            /* output clock is 120MHz */    
            } else {
                CoreClockInput = 0U;                                         /* fc -> reserved */
            }
       } else if (CoreClock == OSC_12M) {                                  /* If input is 16MHz */
            if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_12M_MUL_4_FPLL) {
                CoreClockInput = CoreClock * 4U;                             /* output clock is 48MHz */
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_12M_MUL_10_FPLL) {
                CoreClockInput = CoreClock * 10U;                            /* output clock is 120MHz */               
            } else {
                CoreClockInput = 0U;                                         /* fc -> reserved */
            } 
        } else if (CoreClock == OSC_16M) {                                  /* If input is 16MHz */
            if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_16M_MUL_4_FPLL) {
                CoreClockInput = CoreClock * 4U;                             /* output clock is 64MHz */
            } else if ((TSB_CG->PLLSEL & PLLSEL_MASK) == CG_16M_MUL_6_FPLL) {
                CoreClockInput = CoreClock * 6U;                            /* output clock is 96MHz */               
            } else {
                CoreClockInput = 0U;                                         /* fc -> reserved */
            }
        } else {
            CoreClockInput = 0U;
        }
        
    } else {
        CoreClockInput = CoreClock;
    }
    switch (TSB_CG->SYSCR & 7U) {
    case 0U:
        SystemCoreClock = CoreClockInput;                                    /* Gear -> fc      */
        break;                                                              
    case 1U:
    case 2U:
    case 3U:                                                                 /* Gear -> reserved */
        SystemCoreClock = 0U;
        break;
    case 4U:                                                                 /* Gear -> fc/2     */
        SystemCoreClock = CoreClockInput / 2U;
        break;
    case 5U:                                                                 /* Gear -> fc/4     */
        SystemCoreClock = CoreClockInput / 4U;
        break;
    case 6U:                                                                 /* Gear -> fc/8     */
        SystemCoreClock = CoreClockInput / 8U;
        break;
    case 7U:                                                                 /* Gear -> fc/16     */
        if (CoreClockInput >= OSC_16M){
        SystemCoreClock = CoreClockInput / 16U;
      }else{
      	SystemCoreClock = 0U;
      }
        break;
    default:
        SystemCoreClock = 0U;
        break;
    }
}

/**
 * Initialize the system
 *
 * @param  none
 * @return none
 *
 * @brief  Setup the microcontroller system.
 *         Initialize the System.
 */
void SystemInit(void)
{
#if defined ( __CC_ARM  )/*Enable FPU for Keil*/
  #if (__FPU_USED == 1)                        /* __FPU_USED is defined in core_cm4.h */
    /* enable FPU if available and used */
    SCB->CPACR |= ((3UL << 10*2) |             /* set CP10 Full Access                */
                   (3UL << 11*2)  );           /* set CP11 Full Access                */
  #endif
#endif 

#if (WD_SETUP)                  /* Watchdog Setup */
    TSB_WD->MOD = WDMOD_Val;
    TSB_WD->CR = WDCR_Val;      /*Write Disable code to WDT when WDTE is disabled*/

#endif

#if (CLOCK_SETUP)                           /* Clock(external) Setup */
    TSB_CG->SYSCR = SYSCR_Val;
 
    TSB_CG->OSCCR |= XEN2_1;
    TSB_CG->PLLSEL &= (~PLLON_1);
    TSB_CG->OSCCR &= (~OSCSEL_1);

    TSB_CG->OSCCR |= XEN1_1;
    TSB_CG->OSCCR |= WUPSEL2_1;
    TSB_CG->OSCCR &= WUPSEL1_0;
    TSB_CG->OSCCR &= OSCCR_WUODR_MASK;
    TSB_CG->OSCCR |= OSCCR_WUODR_EXT;
    TSB_CG->OSCCR |= WUEON_1;               /* warm-up timer start */
    while ((TSB_CG->OSCCR & WUEF_1)!= 0U) {   /* 0: warm-up finished */
    }                                       /* Warm-up */
    TSB_CG->OSCCR |= OSCSEL_1;

    TSB_CG->PLLSEL = PLLSEL_Ready;          /* multiply setting */
    TSB_CG->OSCCR |= WUEON_1;               /* warm-up timer start */
    while ((TSB_CG->OSCCR & WUEF_1)!= 0U) {
    }                                       /* Warm-up */
    TSB_CG->PLLSEL |= PLLON_1;
    
    TSB_CG->OSCCR &= OSCCR_WUODR_MASK;
    TSB_CG->OSCCR |= OSCCR_WUODR_PLL;
    
    TSB_CG->STBYCR = STBYCR_Val;
    TSB_CG->OSCCR |= WUEON_1;               /* warm-up timer start */
    while ((TSB_CG->OSCCR & WUEF_1)!= 0U) {
    }                                       /* Warm-up */

    TSB_CG->PLLSEL = PLLSEL_Val;            /* fPLL use */
#endif
}
